<?php

namespace EventPortal\Domain\Event\Models;

use Carbon\Carbon;
use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Concerns\HasTeams;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\Email;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Event extends Model implements AuditableInterface, Statusable
{
    use HasTable,
        AddToFillable,
        AddToCasts,
        Auditable,
        Statable,
        HasStatus,
        HasTeams;

    public const KEY = 'event';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'url',
        'status',
    ];

    protected function getGraph(): string
    {
        return 'event';
    }

    public function agendas(): HasMany
    {
        return $this->hasMany(Agenda::class);
    }

    public function delegates(): BelongsToMany
    {
        return $this
            ->belongsToMany(Delegate::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->as('delegates')
            ->withTimestamps();
    }

    public function emails(): HasMany
    {
        return $this->hasMany(Email::class);
    }

    public function transitionRouteName(): string
    {
        return 'admin.event.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    public function getReadableDatesAttribute(): string
    {
        if ($this->agendas->isEmpty()) {
            return '';
        }

        if ($this->agendas->count() <= 1) {
            return $this->agendas->first()->date->format('d/m/Y');
        }

        return "{$this->agendas->first()->date->format('d/m/Y')} - {$this->agendas->last()->date->format('d/m/Y')}";
    }

    public function scopeHasDateAfterOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '>=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }

    public function scopeHasDateBeforeOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '<=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }
}
